#!/bin/sh
#
# Copyright(C) 2003-2006 ALWIL Software.  All rights reserved.
#
# the avast! 4.7.0 engine library installer
#

#these limits are ~twice bigger than the real needs for the simplest usage scenario
shm_size_limit=32000000
shm_segments_limit=64
shm_segments_probelimit=256
BACKUP_SUFFIX=".backup"

sys_prefix=""
usr_prefix="/usr"
var_prefix="/var"

sysdir="$sys_prefix/etc"
bindir="$usr_prefix/bin"
libdir="$usr_prefix/lib"
docdir="$usr_prefix/share/doc"
mandir="$usr_prefix/share/man"

tempdir="$var_prefix/tmp/avast4"
datadir="$var_prefix/lib/avast4"
moveddir="$var_prefix/lib/avast4/moved"
logdir="$var_prefix/log/avast4"
licdir="$var_prefix/lib/avast4"


# Environment variables
PATH=.:/bin:/usr/bin:/usr/local/bin:/sbin:$PATH
export PATH

##############################
# Subroutines
##############################

platform="Linux"
architecture="i386"
osstyle="other"

# use own install instead of platform-incompatible install script
# for linking, use -l linkname target directory (don't use -m and -l together).
install()
{
    localchmod="no"
    localnonswitch="no"
    locallink="no"
    
while [ $localnonswitch = "no" -a $# -ge 1 ]
do
    case $1 in
        -m)	shift
		localchmod="$1"
		shift
		;;
	-l)	shift
		locallink="yes"
		locallinkname="$1"
		shift
		;;
        -*)  	echo "Install(): Unknown switch '$1' skipped."
		shift
		;;
	*)	localnonswitch="yes"
		;;
    esac
done

if [ $# -eq 2 ]
  then
  if [ $locallink = "yes" ]
    then
        echo "Linking ${locallinkname}->$1 in $2."
	( cd $2; rm -f $locallinkname; ln -s $1 $locallinkname )
    else
	echo "Installing $1 to $2."
	cp  $1 $2
    fi
  if [ $localchmod != "no" ]
    then
	echo "Chmod $localchmod $2."
	chmod $localchmod $2
    fi
else
   echo "Install(): Not enough arguments."
fi
}

testcrontabfail()
  {
  PSH_COMMENT=$PSH_DISABLE
  $1 >/dev/null 2>&1
  if [ $? -ne 0 ]
    then
        echo "# Libavastengine installer: testing of $1 failed - if You want to use this script, fix the problem and enable the line below." >>$PSH_TMP/$PSH_UNIQUE
        PSH_COMMENT="#"
    else
        PSH_DISABLE="#"
    fi
  echo "$PSH_COMMENT 0 0,6,12,18 * * * $PSH_BINDIR/$1" >>$PSH_TMP/$PSH_UNIQUE
  }

# check user (must be root)
check_user()
{
	if [ `id | cut -d= -f2 | cut -d\( -f1` -ne 0 ]; then
		echo ""
		echo "This script must be run as root."
		echo " "
		exit 1
	fi
}

# check OS
check_os()
{
	platform=`uname -s`;
	case $platform in
	Linux)
	 ;;
	FreeBSD)
	 ;;
	SunOS)
	 ;;
	*)
		echo ""
		echo "Your operating system is not supported by the installer."
		echo " "
		exit 1
	 ;;
	esac
	#check os style
	if test -f /etc/init.d/functions
	then
		osstyle="rh"
	elif test -f /etc/rc.status
	then
		osstyle="suse"
	elif test -f /lib/lsb/init-functions
	then
		osstyle="lsb"
	else
		osstyle="other"
	fi
}

# check architecture
check_architecture()
{
	architecture=`uname -m`
	case $architecture in
	  i86pc)
	    ;;
	  i[3456]86)
	    ;;
	  x86_64)
	    ;;
	  sun4u)
	    ;;
	  ppc)
	    ;;
	  *)
		echo ""
		echo "Your architecture ($architecture) is not supported."
		echo " "
		exit 1
	    ;;
	esac
}

##############################
# Main Section
##############################

libname="libavastengine"
librelease="4"
libcurrent="7"
librevision="0"
libage="0"

#check functions
#check_user
check_os
check_architecture

#default variables
avast4vpsreload="avastvpsreload"
avast4vpsupdate="avastvpsupdate.pl"
avast4vpsupdate_alt="avastvpsupdate.sh"
avast4vpsupdate_prefix="avastvpsupdate"
avast4vpsupdateconf="avastvpsupdate.conf"
avast4vpsupdaterc="avastvpsupdaterc"
avast4conffile="avastengine.conf"
avast4rcfile="avastenginerc"
avast4vpsfile="400.vps"
avast4licfile="License.dat"
if [ $platform = "Linux" -o $platform = "SunOS" ]; then
  avast4libname="$libname-$librelease.so.$libcurrent.$libage.$librevision"
  avast4liblink="$libname-$librelease.so.$libcurrent"
else
  avast4libname="$libname-$librelease.so.$libcurrent"
  avast4liblink="$libname-$librelease.so"
fi
avast4liblink2="$libname.so"

#avast4libdir="/usr/lib"
avast4sharedir="/var/lib/avast4"

CRONTABTMP="/tmp/roottab.tmp"
instdir="`dirname "$0"`"


progname=$0
version=0.0.6
packagename="libavastengine"
packagevers="4.7.0"

PSH_CRONTAB="crontab"
PSH_TMP="/var/tmp"
PSH_BINDIR="/usr/bin"
PSH_AVASTVPSUPDATE="avastvpsupdate"
PSH_UNIQUE=`date +%h-%m-%y-%H-%M-%S`-$$


func_usage ()
{
    echo "Usage: $progname [OPTION...]"
    echo "Try '$progname --help' or '$progname -h' for more information."
}

func_help ()
{
    echo "Usage: $progname [OPTION...]"
    echo "$progname -- the avast! engine installer"
    echo ""
    echo "Options:"
    echo "   -p, --prefix          Set installation path"
    echo "   -u, --uninstall       Uninstall previously installed version"
    echo "   -c, --mkconf          Install configuration file for current user"
    echo "   -v, --mkupdateconf    Install VPS updating configuration for current user"
    echo "   -h, --help            Give this help list"
    echo "   -V, --version         Print program version"
    echo ""
    echo "Report bugs to <support@avast.com>."
}

func_version ()
{
    echo "$progname: the avast! engine installer $version"
    echo "Copyright (C) 2003-2006. ALWIL Software. All rights reserved"
}

func_docheck ()
{
    echo "Checking for previous $libname installation"

    check_lib_librarydir=""
    check_man1_vpsreloaddir=""
    check_man1_vpsupdatedir=""
    check_man1_vpsupdatedir_alt=""
    check_man5_engineconfdir=""
    check_bin_vpsreloaddir=""
    check_bin_vpsupdatedir=""
    check_bin_vpsupdatedir_alt=""
    check_doc_dir=""
    check_var_avast4datadir=""
    check_var_avast4tempdir=""
    check_var_avast4moveddir=""
    check_var_avast4logdir=""
    check_var_avast4licensedir=""
    check_var_avast4licensefile=""
    check_lib_librarylink_old=""

    #look for the shared library
    tmp_libdir=""
    case $platform in
    Linux)
       ld_paths=`cat /etc/ld.so.conf`
       ;;
    FreeBSD)
       ld_paths=`cat /etc/defaults/rc.conf | grep -w "ldconfig_paths" | sed -e 's/[^\"]*\"\(.*\)\".*/\1/'`
       ;;
    SunOS)
       ld_paths=`crle | grep "Default Library Path" | cut -d: -f2- | sed s/:/\ /g`
       ;;
    esac
    for x in $libdir $ld_paths /usr/local/lib /usr/lib
    do
	#[ -x $x/$avast4liblink ] && {
	#tmp_libdir=$x;
	if [ -f $x/$avast4libname -a "x$check_lib_librarydir" = x ]
		then
		    check_lib_librarydir=$x;
		fi
	for check_lib_libraryfound in `ls $x/${libname}* 2>/dev/null`
	    do
		if [ -f "$check_lib_libraryfound" -a "$x/$avast4libname" != "$check_lib_libraryfound" ]
		    then
			if [ ! -h "$check_lib_libraryfound" ]
			    then
				if echo "$check_lib_librarylink_old" | grep -v "$check_lib_libraryfound"  >/dev/null 2>&1
				    then
					if echo "$check_lib_libraryfound" | grep -v "$BACKUP_SUFFIX"$ >/dev/null 2>&1
					    then
						check_lib_librarylink_old="$check_lib_librarylink_old $check_lib_libraryfound"
					    fi
				    fi
			    fi
		    fi
	    done
    done
    #if [ -f $tmp_libdir/$avast4libname ]; then check_lib_avast4libpath=$tmp_libdir/avast4libname ; fi

    #check for manual pages
    for x in $mandir `echo $MANPATH | sed 's/:/ /g'` /usr/local/share/man /usr/local/man /usr/share/man
    do
	[ -z "$check_man1_vpsreloaddir" ] && [ -f $x/man1/$avast4vpsreload.1 ] && {
	check_man1_vpsreloaddir=$x/man1;
	}
	[ -z "$check_man1_vpsupdatedir" ] && [ -f $x/man1/$avast4vpsupdate.1 ] && {
	check_man1_vpsupdatedir=$x/man1;
	}
	[ -z "$check_man1_vpsupdatedir_alt" ] && [ -f $x/man1/$avast4vpsupdate_alt.1 ] && {
	check_man1_vpsupdatedir_alt=$x/man1;
	}
	[ -z "$check_man5_engineconfdir" ] && [ -f $x/man5/$avast4conffile.5 ] && {
	check_man5_engineconfdir=$x/man5;
	}
    done

    #check for binary files
    for x in $bindir `echo $PATH | sed 's/:/ /g'`
    do
	[ -z "$check_bin_vpsreloaddir" ] && [ -f $x/$avast4vpsreload ] && {
	check_bin_vpsreloaddir=$x;
	}
	[ -z "$check_bin_vpsupdatedir" ] && [ -f $x/$avast4vpsupdate ] && {
	check_bin_vpsupdatedir=$x;
	}
	[ -z "$check_bin_vpsupdatedir_alt" ] && [ -f $x/$avast4vpsupdate_alt ] && {
	check_bin_vpsupdatedir_alt=$x;
	}
    done

    #check for documentation
    for x in $docdir /usr/shared/doc /usr/local/share/doc
    do
	[ -z "$check_doc_dir" ] && [ -d $x/$packagename-$packagevers ] && {
	check_doc_dir=$x/$packagename-$packagevers;
	}
    done

    conffile_=$sysdir/$avast4conffile
    #look for shared directory
    tmpstr=`cat $conffile_ 2>/dev/null | grep Avast4DataDir | sed 's/.*=//'`
    if [ ! -z "$tmpstr" ]; then
	check_var_avast4datadir=$tmpstr
    fi
    tmpstr=`cat $conffile_ 2>/dev/null | grep Avast4TempDir | sed 's/.*=//'`
    if [ ! -z "$tmpstr" ]; then
	check_var_avast4tempdir=$tmpstr
    fi
    tmpstr=`cat $conffile_ 2>/dev/null | grep Avast4MovedDir | sed 's/.*=//'`
    if [ ! -z "$tmpstr" ]; then
	check_var_avast4moveddir=$tmpstr
    fi
    tmpstr=`cat $conffile_ 2>/dev/null | grep Avast4LogDir | sed 's/.*=//'`
    if [ ! -z "$tmpstr" ]; then
	check_var_avast4logdir=$tmpstr
    fi
    tmpstr=`cat $conffile_ 2>/dev/null | grep Avast4LicenseDir | sed 's/.*=//'`
    if [ ! -z "$tmpstr" ]; then
	check_var_avast4licensedir=$tmpstr
    fi
    tmpstr=`cat $conffile_ 2>/dev/null | grep Avast4LicenseFile | sed 's/.*=//'`
    if [ ! -z "$tmpstr" ]; then
	check_var_avast4licensefile=$tmpstr
    fi
}


  get_agreement() {
    localres=1
    printf "[ yes / no ] "
    read localkey

    case $localkey in
    '') ;;
    [Yy][Ee][Ss]) ;;
    [Oo][Kk]) ;;
    [Yy]) ;;
    [Oo]) ;;
    *)
        localres=0
        ;;
    esac
    return $localres;
  }


func_install ()
{
    ##################
    # Welcome user
    ##################
    echo ""
    echo ""
    echo "Copyright(C) 2003-2006 ALWIL Software.  All rights reserved."
    echo ""
    echo "the avast! engine for $platform will be installed on this machine"
    echo ""
    echo "Technical support is available at:"
    echo "http://www.avast.com/eng/support/technical_support.html"
    echo ""
    echo "To install '$avast4libname' library now, press ENTER."
    echo ""
    echo "To cancel the installation at any time, press Control-C."
    echo ""
    read cont < /dev/tty

    #check for previous library intallation
    func_docheck
    echo ""
    previnstall=0;
    if [ ! -z "$check_lib_librarydir" ] ; then echo "found installed $check_lib_librarydir/$avast4libname file" ; previnstall=1 ;fi
    if [ ! -z "$check_bin_vpsreloaddir" ] ; then echo "found installed $check_bin_vpsreloaddir/$avast4vpsreload file" ; previnstall=1 ;fi
    if [ ! -z "$check_bin_vpsupdatedir" ] ; then echo "found installed $check_bin_vpsupdatedir/$avast4vpsupdate file" ; previnstall=1 ;fi
    if [ ! -z "$check_bin_vpsupdatedir_alt" ] ; then echo "found installed $check_bin_vpsupdatedir_alt/$avast4vpsupdate_alt file" ; previnstall=1 ;fi
    if [ ! -z "$check_man1_vpsreloaddir" ] ; then echo "found installed $check_man1_vpsreloaddir/$avast4vpsreload.1 file" ; previnstall=1 ;fi
    if [ ! -z "$check_man1_vpsupdatedir" ] ; then echo "found installed $check_man1_vpsupdatedir/$avast4vpsupdate.1 file" ; previnstall=1 ;fi
    if [ ! -z "$check_man1_vpsupdatedir_alt" ] ; then echo "found installed $check_man1_vpsupdatedir_alt/$avast4vpsupdate_alt.1 file" ; previnstall=1 ;fi
    if [ ! -z "$check_man5_engineconfdir" ] ; then echo "found installed $check_man5_engineconfdir/$avast4conffile.5 file" ; previnstall=1 ;fi
    if [ ! -z "$check_doc_dir" ] ; then echo "found installed $check_doc_dir directory" ; previnstall=1 ;fi

    foundolder=0;
    if [ ! -z "$check_lib_librarylink_old" ] ; then echo "older avastengine libraries: $check_lib_librarylink_old" ; foundolder=1 ; fi

    if [ $previnstall -eq 1 ] ; then
	echo ""
        echo "Do you want to proceed with the installation?"
	get_agreement
        if [ $? -eq 0 ] ; then
	    echo ""
	    echo "To uninstall previously installed files type: $progname --uninstall"
	    echo ""
	    exit 1
        fi
    fi

    shm_size_avail=`$instdir/bin/shmprobe -s`;
    shm_segments_avail=`$instdir/bin/shmprobe -i"$shm_segments_probelimit"`

    if [ $? -ne 0 ]
        then
            echo ""
            echo "WARNING: Cannot determine SHM limits for Your system!"
        else

            if [ "$shm_size_avail" -lt "$shm_size_limit" -o "$shm_segments_avail" -lt "$shm_segments_limit" ]
                then
                    echo ""
                    echo "WARNING: $avast4libname needs $shm_segments_limit SHM segments, total size $shm_size_limit B."
                    echo "Your system offers $shm_segments_avail SHM segments, total size $shm_size_avail B."
                    echo "(*) On Linux, these limits can be dynamically changed via sysctl variables (w/o reboot):"
                    echo "	kernel.shmall, kernel.shmmax, kernel.shmmni"
                    echo "(*) On old Solaris, add next lines to the /etc/system (these changes require reboot):"
                    echo "	set shmsys:shminfo_shmmax = $shm_size_limit"
                    echo "	set shmsys:shminfo_shmseg = $shm_segments_limit"
                    echo "(*) On Darwin/MacOS X, add next line to the /etc/rc (these changes require reboot):"
                    echo " 	sysctl -w kern.sysv.shmmax=$shm_size_limit kern.sysv.shmmni=$shm_segments_limit kern.sysv.shmall="`expr $shm_size_limit / 4096`" kern.sysv.shmseg=$shm_segments_limit"
		    echo "(*) On FreeBSD, these limits can be dynamically changed via sysctl variables (w/o reboot):"
		    echo "	kern.ipc.shmall, kern.ipc.shmmax, kern.ipc.shmmni, kern.ipc.shmseg"
                    echo ""
                    echo "These changes must be applied before the $avast4libname is loaded,"
		    echo "otherwise the engine migh fail during its initialisation due to"
		    echo "insufficient shared memory resources."
                    echo "Do You want to proceed with the installation (and make changes later)?"

		    get_agreement
                    if [ $? -eq 0 ]
                        then
                            exit 1
			fi
	        fi
        fi

    current_engine_usage=`$instdir/bin/avastvpsreload -n 2>/dev/null | cut -d= -f2`
    current_engine_usage=`echo $current_engine_usage`
    if [ x"$current_engine_usage" != x -a x"$current_engine_usage" != x0 ]
        then
            echo ""
	    echo "WARNING: The $avast4libname seems to be used $current_engine_usage times now."
            echo "All applications which use the engine should be stopped before the new"
            echo "library is installed, otherwise they might crash or fail."
            echo ""
            echo "Do You want to proceed with the installation (and risk the crashes)?"

	    get_agreement
            if [ $? -eq 0 ]
                then
                    exit 1
                fi
        fi

#committing installing paths
    echo ""
    echo "Do you want to install this package to the following directories:"
    echo "    configuration to:  $sysdir"
    echo "    binary files to:   $bindir"
    echo "    shared library to: $libdir"
    echo "    manual pages to:   $mandir"
    echo "    documentation to:  $docdir"
    echo ""
    echo "    The following directories will be used by the shared library:"
    echo "        data dir:    $datadir"
    echo "        temp dir:    $tempdir"
    echo "        log dir:     $logdir"
    echo "        moved dir:   $moveddir"
    echo "        license dir: $licdir"
    echo ""

    get_agreement
    if [ $? -eq 0 ] ; then 
      echo ""
      echo "Please use '--prefix' option to set installation path";
      echo ""
      exit 1
    fi

    #check install paths
    oklibpath=0
    case "$platform" in
    Linux)
       ld_paths=`cat /etc/ld.so.conf`;
       ;;
    FreeBSD)
       ld_paths=`cat /etc/defaults/rc.conf | grep -w "ldconfig_paths" | sed -e 's/[^\"]*\"\(.*\)\".*/\1/'`;
       ;;
    SunOS)
       ld_paths=`crle | grep "Default Library Path" | cut -d: -f2- | sed s/:/\ /g`
       ;;
    esac
    for x in $ld_paths /usr/local/lib /usr/lib
    do
	[ "$x" = "$usr_prefix/lib" ] && {
	oklibpath=1;
	break;
	}
    done

    okbinpath=0
    for x in `echo $PATH | sed -e "s/:/ /g"`
    do
	[ "$x" = "$usr_prefix/bin" ] && {
	okbinpath=1
	break;
	}
    done

    if [ $oklibpath -eq 0 ]||[ $okbinpath -eq 0 ] ; then echo ""; fi
    if [ $oklibpath -eq 0 ]; then echo "WARNING: our system doesn't support $usr_prefix/lib library path"; fi
    if [ $okbinpath -eq 0 ]; then echo "WARNING: our system doesn't support $usr_prefix/bin executable path"; fi

    if [ $oklibpath -eq 0 ]||[ $okbinpath -eq 0 ] ; then
	echo ""
        echo "Do you want to proceed with the installation?"
	get_agreement
        if [ $? -eq 0 ] ; then
	    echo ""
	    exit 1
        fi
    fi

    #installing the library
    if [ ! -d $libdir ]; then mkdir -p $libdir ; fi
    install -m 755 "$instdir/lib/$avast4libname" "$libdir/$avast4libname" ;
    for liblinks in `( cd ${instdir}/lib; ls )`
      do
    #install() is capable of linking
	if ls -l ${instdir}/lib/$liblinks | grep "^l" >/dev/null 2>&1
	  then
    	    install  -l $liblinks $avast4libname $libdir
	  fi
      done

    if [ -h $libdir/libavastengine.so.4 ]; then
	( cd $libdir ; rm -f libavastengine.so.4; ln -s $avast4libname libavastengine.so.4 )
    fi
    if [ -f $libdir/libavastengine.so.4.5 ]; then
	( cd $libdir ; rm -f libavastengine.so.4.5; ln -s $avast4libname libavastengine.so.4.5 )
    fi

    if [ ! -f $libdir/$avast4libname ] ; then
	echo ""
	echo "Intallation failed!"
	echo ""
	exit 1
    fi

    #installing executable files
    if [ ! -d $bindir ]; then mkdir -p $bindir ; fi
    install -m 755 "$instdir/bin/$avast4vpsreload" "$bindir/$avast4vpsreload"
    install -m 755 "$instdir/bin/$avast4vpsupdate" "$bindir/$avast4vpsupdate"
    install -m 755 "$instdir/bin/$avast4vpsupdate_alt" "$bindir/$avast4vpsupdate_alt"

    #installing manual pages
    if [ ! -d $mandir/man1 ]; then mkdir -p $mandir/man1 ; fi
    install -m 644 "$instdir/man/man1/$avast4vpsreload.1" "$mandir/man1/$avast4vpsreload.1"
    install -m 644 "$instdir/man/man1/$avast4vpsupdate.1" "$mandir/man1/$avast4vpsupdate.1"
    install -m 644 "$instdir/man/man1/$avast4vpsupdate_alt.1" "$mandir/man1/$avast4vpsupdate_alt.1"
    if [ ! -d $mandir/man5 ]; then mkdir -p $mandir/man5 ; fi
    install -m 644 "$instdir/man/man5/$avast4conffile.5" "$mandir/man5/$avast4conffile.5"

    #installing documentation
    if [ ! -d $docdir/$packagename-$packagevers ]; then mkdir -p $docdir/$packagename-$packagevers ; fi
    install -m 644 "$instdir/AUTHORS" "$docdir/$packagename-$packagevers/AUTHORS"
    install -m 644 "$instdir/ChangeLog" "$docdir/$packagename-$packagevers/ChangeLog"
    install -m 644 "$instdir/INSTALL" "$docdir/$packagename-$packagevers/INSTALL"
    install -m 644 "$instdir/LICENSE" "$docdir/$packagename-$packagevers/LICENSE"
    install -m 644 "$instdir/MANIFEST" "$docdir/$packagename-$packagevers/MANIFEST"
    install -m 644 "$instdir/README" "$docdir/$packagename-$packagevers/README"
    install -m 644 "$instdir/FAQ" "$docdir/$packagename-$packagevers/FAQ"

    #install shared file
    mkdir -p "$datadir" ; chmod 0755 "$datadir"
    install -m 644 "$instdir/var/$avast4vpsfile" "$datadir/$avast4vpsfile"
    mkdir -p "$tempdir" ; chmod 1777 "$tempdir"
    mkdir -p "$logdir" ; chmod 0755 "$logdir"
    mkdir -p "$moveddir" ; chmod 1777 "$moveddir"
    mkdir -p "$licdir" ; chmod 0755 "$licdir"
    #install -m 644 "$instdir/var/$avast4licfile" "$datadir/$avast4licfile"

    #installing etc files
    if [ ! -d $sysdir ]; then mkdir -p $sysdir ; fi
    if [ -f $sysdir/$avast4conffile ]; then mv "$sysdir/$avast4conffile" "$sysdir/$avast4conffile.old" ; fi
    install -m 644 "$instdir/etc/$avast4conffile" "$sysdir/$avast4conffile"
    if [ -f $sysdir/$avast4vpsupdateconf ]; then mv "$sysdir/$avast4vpsupdateconf" "$sysdir/$avast4vpsupdateconf.old" ; fi
    install -m 644 "$instdir/etc/$avast4vpsupdateconf" "$sysdir/$avast4vpsupdateconf"

### point links to the old engine

#
# backward binary compatibility should be back ;)
#

if [ $foundolder -ne 0 ]
    then
echo
echo "WARNING: Older version of libavastengine was found:" 
echo "$check_lib_librarylink_old"
echo "Thanx to the backward compatibility it's possible to link"
echo "these files to the current library. Corresponding older"
echo "versions of utilities such as avastcmd or avastd won't be"
echo "able to use the new engine without this link. The older"
echo "library will be kept with the \"$BACKUP_SUFFIX\" suffix"
echo "in the same directory."

echo
echo "Do you want to backup the old library and create link to"
echo "the new library?"
	get_agreement
        if [ $? -ne 0 ] ; then
	    for check_lib_librarylink_current in $check_lib_librarylink_old
		do
		    mv $check_lib_librarylink_current ${check_lib_librarylink_current}$BACKUP_SUFFIX
		    ln -s $libdir/$avast4liblink $check_lib_librarylink_current
		done
        fi
fi

### update crontab

echo
echo "Installer will insert call for virus database update to the crontab now."
echo "This task can be done by $PSH_AVASTVPSUPDATE.sh or $PSH_AVASTVPSUPDATE.pl."
echo "Installer will check which way is suitable for you, and also will try"
echo "to download the latest database version. This step might take up to few"
echo "minutes under some circumstances, but can be interrupted anytime by"
echo "pressing Ctrl-C (in this case, add the line to the crontab manually"
echo "and probably fix the problem in /etc/avastvpsupdate.conf)."

### try to update the vps

$PSH_CRONTAB -l | grep -v $PSH_AVASTVPSUPDATE >$PSH_TMP/$PSH_UNIQUE

echo "# Line below polls for Avast! update - running $PSH_AVASTVPSUPDATE.pl or $PSH_AVASTVPSUPDATE.sh periodically does the job." >>$PSH_TMP/$PSH_UNIQUE

PSH_DISABLE=""

testcrontabfail $PSH_AVASTVPSUPDATE.pl
testcrontabfail $PSH_AVASTVPSUPDATE.sh

$PSH_CRONTAB $PSH_TMP/$PSH_UNIQUE

###
#    echo "There are two similar scripts for automatical update"
#    echo "avastvpsupdate.pl and avastvpsupdate.sh. The default"
#    echo "Perl script depends on the MD5 and URI modules, the"
#    echo "alternative shell script depends on md5sum and wget"
#    echo "or lynx. You can change the scripts later in your"
#    echo "crontab file."
#    echo ""
#    echo "Do you want to use the alternative shell script?"
    
#    get_agreement
#    if [ $? -ne 0 ]
#	then
#	    avast4vpsupdate=$avast4vpsupdate_alt
#	fi
    

    #set crontab item for the avastvpsupdate.pl script
#    crontab -l 2>/dev/null | grep -v "# DO NOT EDIT THIS FILE -" | grep -v "# (${CRONTABTMP}" | grep -v "# (Cron version --" | grep -v "* $bindir/$avast4vpsupdate" > ${CRONTABTMP}
#    crontabline="0 0,6,12,18 * * * $bindir/$avast4vpsupdate"
#    echo "Updating crontab, line added:"
#    echo "$crontabline"
#    echo "$crontabline" >>$CRONTABTMP
#    crontab ${CRONTABTMP}
#    rm -f ${CRONTABTMP}

    #update cron daemon
    if [ $platform != "SunOS" ]
      then
	pid=`ps a | grep crond | awk '{print $1}'`
	if [ "${pid}" = "0" ]; then
	    kill -HUP ${pid}
	fi
    fi

    sleep 1
    echo ""
    echo "The $avast4libname library installation complete."
    echo ""
    echo "***NOTE***: this script has set the cron daemon to check for the new VPS"
    echo "file periodicaly after six hours. You can change this period by using"
    echo "'crontab -e' command. The configuration for updating VPS is available"
    echo "in $sysdir/$avast4vpsupdateconf file."
    echo "Make sure Digest::MD5, File::Spec, LWP::UserAgent and URI Perl modules"
    echo "are installed before running '$bindir/avastvpsupdate.pl' script."
    echo "Make sure wget (or lynx) and md5sum (or md5) utilities"
    echo "are installed before running '$bindir/avastvpsupdate.sh' script."
    echo ""
    echo "***NOTE***: the installation package does not contain a license file."
    echo "Any program using this library will not work until you will have a valid"
    echo "license file (with either a DEMO or a RELEASE license)."
    echo "To request a DEMO license, please visit the following web page:"
    echo "http://www.avast.com/i_kat_340.php?lang=ENG"
    echo ""
}

#install user configuration
func_makeuserconf ()
{
    if [ -z "$1" ]
    then
      if [ $platform = "SunOS" ]
        then
	  userhomedir=`(cd; pwd)`
	else
          userhomedir=`(cd ~;pwd)`;
	fi
    else
	userhomedir=$1;
    fi

    if [ $platform = "SunOS" ]
      then
        localwhoami=`id | cut -d\) -f1 | cut -d\( -f2`
      else
        localwhoami=`whoami`
      fi

    echo ""
    echo ""
    echo "Copyright(C) 2003-2006 ALWIL Software.  All rights reserved."
    echo ""
    echo "the avast! engine for $platform configuration file for $localwhoami user"
    echo "will be installed in $userhomedir directory."
    echo ""
    echo "To install '$userhomedir/.$avast4rcfile' configuration file now, press ENTER."
    echo ""
    echo "To cancel the installation at any time, press Control-C."
    echo ""
    read cont < /dev/tty

    tempdir_home="$userhomedir/.avastengine/tmp"
    datadir_home="$userhomedir/.avastengine"
    moveddir_home="$userhomedir/.avastengine/moved"
    logdir_home="$userhomedir/.avastengine/log"
    licdir_home="$userhomedir/.avastengine"

    echo ""
    echo "The following directories will be used by the shared library:"
    echo "    data dir:    $datadir_home"
    echo "    temp dir:    $tempdir_home"
    echo "    log dir:     $logdir_home"
    echo "    moved dir:   $moveddir_home"
    echo "    license dir: $licdir_home"
    echo ""

    mkdir -p "$userhomedir/.avastengine" ; chmod 0700 "$userhomedir/.avastengine"

    mkdir -p "$datadir_home" ; chmod 0700 "$datadir_home"
    install -m 644 "$instdir/var/$avast4vpsfile" "$datadir_home/$avast4vpsfile"
    mkdir -p "$tempdir_home" ; chmod 0700 $tempdir_home
    mkdir -p "$logdir_home" ; chmod 0700 $logdir_home
    mkdir -p "$moveddir_home" ; chmod 0700 $moveddir_home
    mkdir -p "$licdir_home" ; chmod 0700 $licdir_home
    #install -m 644 "$instdir/var/$avast4licfile" "$datadir/$avast4licfile"

    #installing etc files
    if [ -f $userhomedir/.$avast4rcfile ]; then mv "$userhomedir/.$avast4rcfile" "$userhomedir/.$avast4rcfile.old" ; fi
    #update configuration paths
    cat $instdir/etc/$avast4conffile | while read line
    do
	case `echo $line | sed 's/=.*//'` in
        Avast4DataDir ) echo "Avast4DataDir=$datadir_home" >> $userhomedir/.$avast4rcfile ;;
	Avast4TempDir ) echo "Avast4TempDir=$tempdir_home" >> $userhomedir/.$avast4rcfile ;;
        Avast4MovedDir ) echo "Avast4MovedDir=$moveddir_home" >> $userhomedir/.$avast4rcfile ;;
	Avast4LogDir ) echo "Avast4LogDir=$logdir_home" >> $userhomedir/.$avast4rcfile ;;
        Avast4LicenseDir ) echo "Avast4LicenseDir=$licdir_home" >> $userhomedir/.$avast4rcfile ;;
	Avast4LicenseFile ) echo "Avast4LicenseFile=$licdir_home/$avast4licfile" >> $userhomedir/.$avast4rcfile ;;
        * ) echo "$line" >> $userhomedir/.$avast4rcfile ;;
	esac
    done
    chmod 644 $userhomedir/.$avast4rcfile

    echo ""
    echo "The configuration file installed!"
    echo ""
    echo "To keep your VPS file fresh, please, install '~/.$avast4vpsupdaterc'"
    echo "file (by running './mkinstall.sh -v' command) and then setup the cron"
    echo "daemon to periodically check for a new version."
    echo ""
}

func_makeuserupdateconf ()
{

    if [ -z "$1" ]
    then
      if [ $platform = "SunOS" ]
        then
	  userhomedir=`(cd; pwd)`
	else
          userhomedir=`(cd ~;pwd)`;
	fi
    else
	userhomedir=$1;
    fi

    if [ $platform = "SunOS" ]
      then
        localwhoami=`id | cut -d\) -f1 | cut -d\( -f2`
      else
        localwhoami=`whoami`
      fi

    echo ""
    echo ""
    echo "Copyright(C) 2003-2006 ALWIL Software.  All rights reserved."
    echo ""
    echo "the avast! VPS updating configuration file for $localwhoami user"
    echo "will be installed in $userhomedir directory."
    echo ""
    echo "To install '$userhomedir/.$avast4vpsupdaterc' configuration file now, press ENTER."
    echo ""
    echo "To cancel the installation at any time, press Control-C."
    echo ""
    read cont < /dev/tty

    if [ -f $userhomedir/.$avast4vpsupdaterc ]; then mv "$userhomedir/.$avast4vpsupdaterc" "$userhomedir/.$avast4vpsupdaterc.old" ; fi
    cat $instdir/etc/$avast4vpsupdateconf > $userhomedir/.$avast4vpsupdaterc
    chmod 644 $userhomedir/.$avast4vpsupdaterc

    echo ""
    echo "The configuration file installed!"
    echo ""
    echo "To keep your VPS file up to date, please, run 'crontab -e' to edit"
    echo "crontab table to set 'avastvpsupdate.pl' script to be running by the"
    echo "cron daemon in user defined period. Please, consult crontab manual page"
    echo "to know, how to do it (man -S 1 crontab)."
    echo "To check for the VPS file every sixth hour, please insert into crontab:"
    echo "0 0,6,12,18 * * * $bindir/$avast4vpsupdate"
    echo ""
}

func_uninstall ()
{
    echo ""
    echo ""
    echo "Copyright(C) 2003-2006 ALWIL Software.  All rights reserved."
    echo ""
    echo "To uninstall '$avast4libname' library now, press ENTER."
    echo ""
    echo "To cancel the installation at any time, press Control-C."
    echo ""
    read cont < /dev/tty

    #check for previous library intallation
    func_docheck
    echo ""
    previnstall=0;
    if [ ! -z "$check_lib_librarydir" ] ; then echo "found installed $check_lib_librarydir/$avast4libname file" ; previnstall=1 ; fi
    if [ ! -z "$check_bin_vpsreloaddir" ] ; then echo "found installed $check_bin_vpsreloaddir/$avast4vpsreload file" ; previnstall=1 ;fi
    if [ ! -z "$check_bin_vpsupdatedir" ] ; then echo "found installed $check_bin_vpsupdatedir/$avast4vpsupdate file" ; previnstall=1 ;fi
    if [ ! -z "$check_bin_vpsupdatedir_alt" ] ; then echo "found installed $check_bin_vpsupdatedir_alt/$avast4vpsupdate_alt file" ; previnstall=1 ;fi
    if [ ! -z "$check_man1_vpsreloaddir" ] ; then echo "found installed $check_man1_vpsreloaddir/$avast4vpsreload.1 file" ; previnstall=1 ;fi
    if [ ! -z "$check_man1_vpsupdatedir" ] ; then echo "found installed $check_man1_vpsupdatedir/$avast4vpsupdate.1 file" ; previnstall=1 ;fi
    if [ ! -z "$check_man1_vpsupdatedir_alt" ] ; then echo "found installed $check_man1_vpsupdatedir_alt/$avast4vpsupdate_alt.1 file" ; previnstall=1 ;fi
    if [ ! -z "$check_man5_engineconfdir" ] ; then echo "found installed $check_man5_engineconfdir/$avast4conffile.5 file" ; previnstall=1 ;fi
    if [ ! -z "$check_doc_dir" ] ; then echo "found installed $check_doc_dir directory" ; previnstall=1 ;fi

    if [ $previnstall -eq 1 ] ; then
	echo ""
        echo "Do you want to uninstall these files?"
	get_agreement
        if [ $? -eq 0 ] ; then
    	    exit 1
	else
	    if [ ! -z "$check_lib_librarydir" ] ; then
		rm -f $check_lib_librarydir/$avast4libname ;
		rm -f $check_lib_librarydir/$avast4liblink ;
		rm -f $check_lib_librarydir/$avast4liblink2 ;
	    fi
	    if [ ! -z "$check_bin_vpsreloaddir" ] ; then rm -f $check_bin_vpsreloaddir/$avast4vpsreload ; fi
	    if [ ! -z "$check_bin_vpsupdatedir" ] ; then rm -f $check_bin_vpsupdatedir/$avast4vpsupdate ; fi
	    if [ ! -z "$check_bin_vpsupdatedir_alt" ] ; then rm -f $check_bin_vpsupdatedir_alt/$avast4vpsupdate_alt ; fi
	    if [ ! -z "$check_man1_vpsreloaddir" ] ; then rm -f $check_man1_vpsreloaddir/$avast4vpsreload.1 ; fi
	    if [ ! -z "$check_man1_vpsupdatedir" ] ; then rm -f $check_man1_vpsupdatedir/$avast4vpsupdate.1 ; fi
	    if [ ! -z "$check_man1_vpsupdatedir_alt" ] ; then rm -f $check_man1_vpsupdatedir_alt/$avast4vpsupdate_alt.1 ; fi
	    if [ ! -z "$check_man5_engineconfdir" ] ; then rm -f $check_man5_engineconfdir/$avast4conffile.5 ; fi
	    if [ ! -z "$check_doc_dir" ] ; then rm -fr $check_doc_dir 2>/dev/null ; fi

	    #400.vps file
	    if [ ! -z "$check_var_avast4datadir" ] ; then rm -f $check_var_avast4datadir/$avast4vpsfile ; fi

	    #uninstall crontab item for the avastvpsupdate.pl script
	    
	    $PSH_CRONTAB -l | grep -v $PSH_AVASTVPSUPDATE >$PSH_TMP/$PSH_UNIQUE

	    $PSH_CRONTAB $PSH_TMP/$PSH_UNIQUE

	    
#	    if crontab -l | grep "* $check_bin_vpsupdatedir/$avast4vpsupdate_prefix" >/dev/null 2>&1
#	       then
#		rm -f ${CRONTABTMP}
#		crontab -l 2>/dev/null | grep -v "# DO NOT EDIT THIS FILE -" | grep -v "# (${CRONTABTMP}" | grep -v "# (Cron version --" | grep -v "* $check_bin_vpsupdatedir/$avast4vpsupdate_prefix" >${CRONTABTMP}
#		if [ -f ${CRONTABTMP} ]; then
#		    crontab ${CRONTABTMP}
#		    rm -f ${CRONTABTMP}
#		else
#		    crontab -d > /dev/null 2>&1
#		    if [ $? -ne 0 ]; then
#			echo "y" | crontab -r > /dev/null 2>&1
#		    fi
#		fi

		# update cron daemon
		if [ $platform != "SunOS" ]
		then
		    pid=`ps a | grep crond | awk '{print $1}'`
		    if [ "${pid}" = "0" ]; then
			kill -HUP ${pid}
		    fi
		fi
#	    fi

	    echo ""
	    echo "Uninstallation complete!"
	    echo ""
	fi
    else
	echo ""
	echo "Could not find previously installed files !!!"
	echo ""
	exit 1
    fi
}


while test "$#" -gt 0
do
  arg="$1"
  shift

  case $arg in
      -*=*) optarg=`echo "$arg" | sed -e 's/[-_a-zA-Z0-9]*=//'` ;;
      *) optarg= ;;
  esac

  case "$arg" in
      --prefix=* | -p=* )
	  if [ $optarg = "/usr/local" ]; then sys_prefix=$optarg ; fi
	  usr_prefix=$optarg;

	  sysdir="$sys_prefix/etc"
	  bindir="$usr_prefix/bin"
	  libdir="$usr_prefix/lib"
	  docdir="$usr_prefix/share/doc"
	  mandir="$usr_prefix/share/man"
	  if [ "$usr_prefix" = "/usr/local" ] && [ ! `uname -s` = "Linux" ]; then
	      mandir="$usr_prefix/man";
          fi

	  #tempdir="$var_prefix/tmp/avast4"
	  #datadir="$var_prefix/lib/avast4"
	  #moveddir="$var_prefix/lib/avast4/moved"
	  #logdir="$var_prefix/lib/avast4/log"
	  #licdir="$var_prefix/lib/avast4"

	  ;;
      --uninstall | -u )
	  check_user
	  func_uninstall; exit 0;;
      --mkconf | -c )
	  func_makeuserconf; exit 0 ;;
      --mkupdateconf | -v )
	  func_makeuserupdateconf; exit 0 ;;
      --help | -h )
	  func_help; exit 0 ;;
      --version | -V )
	  func_version; exit 0 ;;
  esac
done
#func_usage 1>&2
check_user
func_install

exit 0
